mc_core::feetech

Struct Bus

pub struct Bus { /* private fields */ }
servo bus

Implementations

Source

impl Bus

pub fn new(port: SerialStream, path: &Path) -> Self

Create new Feetech servo bus.

Arguments
  • port - serial port to use for communication with the servo bus
  • path - path to the serial port
Returns

new servo bus

pub async fn send_ping(&mut self, id: usize) -> Result<()>

Send ping to one of the servos.

Arguments
  • id - ID of the servo to ping
Returns

Ok(()) if the ping was sent successfully, Err(e) otherwise

pub async fn recv_ping(&mut self) -> Result<usize>

Receive response from previously sent ping.

Returns

Ok(id) if the response was received from the servo with the given ID, Err(e) otherwise

pub async fn send_ping_all(&mut self) -> Result<()>

Send ping to all servos.

Returns

Ok(()) if the ping was sent successfully, Err(e) otherwise

pub async fn recv_ping_all(&mut self) -> Result<Vec<usize>>

Receive responses from previously sent ping to all servos.

Returns

Ok(ids) with ids being a vector of IDs of the servos that responded, Err(e) otherwise

pub async fn send_write( &mut self, id: usize, start: Register, data: &[u8], ) -> Result<()>

Send write command to one of the servos.

Arguments
  • id - ID of the servo to write to
  • start - starting register to write to
  • data - data to write to the servo
Returns

Ok(()) if the write was sent successfully, Err(e) otherwise

pub async fn send_read( &mut self, id: usize, start: Register, length: u8, ) -> Result<()>

Send read command to one of the servos.

Arguments
  • id - ID of the servo to read from
  • start - starting register to read from
  • length - number of bytes to read
Returns

Ok(()) if the read was sent successfully, Err(e) otherwise

pub async fn recv_read(&mut self, length: usize) -> Result<Vec<u8>>

Receive response from previously sent read command.

Arguments
  • length - number of bytes to read
Returns

Ok(data) with data being a vector of bytes read from the servo, Err(e) otherwise

pub async fn send_sync_write( &mut self, start: Register, data: HashMap<usize, Vec<u8>>, ) -> Result<()>

Send write command to multiple servos.

Arguments
  • start - starting register to write to
  • data - map of servo IDs to data to write to the servo
Returns

Ok(()) if the write was sent successfully, Err(e) otherwise

pub async fn send_sync_read( &mut self, start: Register, length: u8, ids: &[usize], ) -> Result<()>

Send read command to multiple servos.

Arguments
  • start - starting register to read from
  • length - number of bytes to read
  • ids - vector of IDs of the servos to read from
Returns

Ok(()) if the read was sent successfully, Err(e) otherwise

pub async fn recv_sync_read( &mut self, count: usize, length: usize, ) -> Result<HashMap<u8, Vec<u8>>>

Receive responses from previously sent read command to multiple servos.

Arguments
  • count - number of servos to read from
  • length - number of bytes to read per servo
Returns

Ok(data) with data being a map of servo IDs to data read from the servo, Err(e) otherwise

pub fn port_path(&self) -> &Path

Get path to the serial port.

Returns

path to the serial port

pub fn add_servo(&mut self, id: usize, params: &ServoParams)

Add servo to the bus.

Arguments
  • id - ID of the servo to add
  • params - parameters of the servo
pub fn remove_servo(&mut self, id: usize)

Remove servo from the bus.

Arguments
  • id - ID of the servo to remove
pub async fn reset_servo(&mut self, id: usize) -> Result<()>

Reset servo and make the current position 0.

Arguments
  • id - ID of the servo to reset
Returns

Ok(()) if the servo was reset successfully, Err(e) otherwise

pub fn set_servo_params( &mut self, id: usize, params: &ServoParams, ) -> Result<()>

Set servo parameters.

Arguments
  • id - ID of the servo to set parameters for
  • params - parameters to set
Returns

Ok(()) if the servo parameters were set successfully, Err(e) otherwise

pub fn servo_state(&self, id: usize) -> Option<&ServoState>

Get servo state reference.

Arguments
  • id - ID of the servo to get state for
Returns

Some(state) if the servo state was found, None otherwise

pub fn servo_goals_mut(&mut self, id: usize) -> Option<&mut ServoGoals>

Get mutable servo goals reference.

Arguments
  • id - ID of the servo to get goals for
Returns

Some(goals) if the servo goals were found, None otherwise

pub fn servo_params_mut(&mut self, id: usize) -> Option<&mut ServoParams>

Get mutable servo parameters reference.

Arguments
  • id - ID of the servo to get parameters for
Returns

Some(params) if the servo parameters were found, None otherwise

pub async fn init_loop(&mut self) -> Result<()>

Initialize loop. This is called once to request all servos to return their current state.

Returns

Ok(()) if the loop was initialized successfully, Err(e) otherwise

pub async fn update_recv(&mut self) -> Result<()>

Process servo state responses.

Returns

Ok(()) if the loop was updated successfully, Err(e) otherwise

pub async fn update_send(&mut self) -> Result<()>

Send new goals to the servos.

Returns

Ok(()) if the goals were sent successfully, Err(e) otherwise